# SPDX-License-Identifier: GPL-2.0-or-later

import bpy
from bpy.types import Panel
from pathlib import Path
from vse_toolbox.bl_utils import (get_addon_prefs, get_scene_settings, get_strip_settings)
from vse_toolbox.constants import ASSET_PREVIEWS
from vse_toolbox.sequencer_utils import (set_active_strip, get_channel_name)


class VSETB_main:
    bl_space_type = "SEQUENCE_EDITOR"
    bl_region_type = "UI"
    bl_category = "VSE ToolBox"
    bl_label = "VSE ToolBox"


class VSETB_PT_main(VSETB_main, Panel):

    def draw_header_preset(self, context):
        self.layout.operator('vse_toolbox.load_projects', icon='FILE_REFRESH', text='', emboss=False)

    def draw(self, context):
        wm = context.window_manager
        scn = context.scene

        settings = get_scene_settings()
        prefs = get_addon_prefs()
        
        project = settings.active_project

        layout = self.layout
        col = layout.column()

        col.prop(settings, 'project_name', text='Project')

        if project:
            if project.type == 'TVSHOW':
                col.prop(project, 'episode_name', text='Episodes')

        #col.separator()

        #row = col.row(align=True)
        
        #row.prop(settings, 'toogle_prefs', text='', icon='PREFERENCES', toggle=True)
        
        '''
        if settings.toogle_prefs:
            box = col.box()
            col = box.column(align=True)
            col.use_property_split = True
            col.use_property_decorate = False
            
            col.prop(settings, 'project_name', text='Projects')

            if project:
                if project.type == 'TV Shows':
                    col.prop(project, 'episode_name', text='Episodes')
                
                #col.prop(project, 'sequence_template')
                #col.prop(project, 'shot_template')
                # col.separator()
                # col.operator('vse_toolbox.new_episode', text='Add Episode', icon='IMPORT')
        '''

        # Rename


class VSETB_PT_sequencer(VSETB_main, Panel):
    bl_label = "Sequencer"
    bl_parent_id = "VSETB_PT_main"
    
    def draw_header_preset(self, context):
        settings = get_scene_settings()
        ico = ("RESTRICT_SELECT_OFF" if settings.auto_select_strip else "RESTRICT_SELECT_ON")
        self.layout.prop(settings, "auto_select_strip", text="", icon=ico)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        col = layout.column()

        col.operator('vse_toolbox.set_sequencer', text='Set-Up Sequencer', icon='SEQ_SEQUENCER')

        #row = col.row()
        shot_label = ''
        sequence_label = ''

        if project:
            episode = project.episode_name
            sequence_label = project.sequence_template.format(episode=episode, index=project.sequence_start_number)
            shot_label = project.shot_template.format(episode=episode, sequence=sequence_label, index=project.shot_start_number)
        
        #row.separator()

        strip = context.active_sequence_strip
        channel_name = get_channel_name(strip) or ''
        if channel_name == 'Shots':
            label = shot_label
        elif channel_name == 'Sequences':
            label = sequence_label
        else:
            label = 'Not Supported'

        row = col.row(align=True)
        if label == 'Not Supported':
            row.enabled = False

        op = row.operator('vse_toolbox.strips_rename', text=f'Rename {channel_name}  ( {label} )', icon='SORTALPHA')
        if project:
            op.channel_name = channel_name
            if channel_name == 'Shots':
                op.start_number = project.shot_start_number
                op.template = project.shot_template
                op.increment = project.shot_increment
            else:
                op.start_number = project.sequence_start_number
                op.template = project.sequence_template
                op.increment = project.sequence_increment
        
        col.operator('vse_toolbox.set_stamps', text='Set Stamps', icon='COLOR')


class VSETB_PT_settings(VSETB_main, Panel):
    bl_label = "Settings"
    bl_parent_id = "VSETB_PT_main"
    bl_options = {'DEFAULT_CLOSED'}
    
    #def draw_header_preset(self, context):
    #    self.layout.operator('vse_toolbox.import_files', icon='IMPORT', text='', emboss=False)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        col = layout.column()
        #row = col.row(align=True)
        col.prop(project, 'sequence_template')
        col.prop(project, 'shot_template')
        col.prop(project, 'render_template')


class VSETB_PT_imports(VSETB_main, Panel):
    bl_label = "Imports"
    bl_parent_id = "VSETB_PT_main"
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw_header_preset(self, context):
        self.layout.operator('vse_toolbox.import_files', icon='IMPORT', text='', emboss=False)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        col = layout.column()
        #row = col.row(align=True)
        col.operator('vse_toolbox.import_files', text='Import', icon='IMPORT')


class VSETB_PT_exports(VSETB_main, Panel):
    bl_label = "Exports"
    bl_parent_id = "VSETB_PT_main"
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw_header_preset(self, context):
        self.layout.operator('vse_toolbox.export_csv', icon='EXPORT', text='', emboss=False)

    def draw(self, context):
        prefs = get_addon_prefs()
        layout = self.layout
        settings = get_scene_settings()
        project = settings.active_project

        # TODO FAIRE DES VRAIS OPS
        layout.operator('vse_toolbox.strips_render', text='Render Strips', icon='SEQUENCE')

        tracker_label = settings.tracker_name.title().replace('_', ' ')
        layout.operator('vse_toolbox.upload_to_tracker', text=f'Upload to {tracker_label}', icon='EXPORT')
        layout.operator('vse_toolbox.export_csv', text='Export csv', icon='SPREADSHEET')


class VSETB_PT_casting(VSETB_main, Panel):
    bl_label = "Casting"
    bl_parent_id = "VSETB_PT_main"

    def draw_header_preset(self, context):
        active_strip = context.scene.sequence_editor.active_strip
        self.layout.label(text=active_strip.name)

    @classmethod
    def poll(cls, context):
        strip = context.scene.sequence_editor.active_strip
        return strip and get_channel_name(strip) == 'Shots'

    def draw(self, context):
        layout = self.layout
        
        settings = get_scene_settings()
        strip_settings = get_strip_settings()

        project = settings.active_project

        if not project:
            return

        if not project.assets:
            row = layout.row(align=True)
            row.label(text='No Assets in this Project')
        else:

            row = layout.row()
            col = row.column()
            col.template_list("VSETB_UL_casting", "shot_casting", strip_settings, "casting", strip_settings, "casting_index", rows=6)
            
            if strip_settings.casting:
                casting_item = strip_settings.casting[strip_settings.casting_index]
                asset = casting_item.asset
                if asset:
                    ico = ASSET_PREVIEWS.get(asset.preview)            
                    if ico:
                        box = col.box()
                        box.template_icon(icon_value=ico.icon_id, scale=7.5)
                    
            col_tool = row.column(align=True)
            col_tool.operator('vse_toolbox.casting_add', icon='ADD', text="")
            col_tool.operator('vse_toolbox.casting_remove', icon='REMOVE', text="")
            col_tool.separator()
            col_tool.operator('vse_toolbox.casting_move', icon='TRIA_UP', text="").action = 'UP'
            col_tool.operator('vse_toolbox.casting_move', icon='TRIA_DOWN', text="").action = 'DOWN'
            col_tool.separator()
            col_tool.operator('vse_toolbox.copy_casting', icon='COPYDOWN', text="")
            col_tool.operator('vse_toolbox.paste_casting', icon='PASTEDOWN', text="")


class VSETB_PT_metadata(VSETB_main, Panel):
    bl_label = "Shot Metadata"
    bl_parent_id = "VSETB_PT_casting"
    #bl_options = {"DEFAULT_CLOSED"}

    def draw(self, context):
        layout = self.layout
        scn = context.scene
        active_strip = scn.sequence_editor.active_strip
        
        if not active_strip:
            return 

        settings = get_scene_settings()
        strip_settings = get_strip_settings()

        project = settings.active_project

        if not project:
            return

        #col = layout.column()
        for key in strip_settings.metadata.__annotations__.keys():
            layout.prop(strip_settings.metadata, key, text=key.upper())

classes = (
    VSETB_PT_main,
    VSETB_PT_imports,
    VSETB_PT_sequencer,
    VSETB_PT_casting,
    VSETB_PT_metadata,
    VSETB_PT_exports,
)

def register(): 
    for cls in classes:
        bpy.utils.register_class(cls)

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)